# Package modules
from nosypy import autoencoder
from nosypy import data
from nosypy import run 
from nosypy import annotate

# For nested packages 
from nosypy import image 
from nosypy import novelty 
# from nosypy import visualise 

__version__ = '0.0.2'
name = 'nosypy'